@@ -1,7 +1,7 @@
-from dataclasses import dataclass
 from dataclasses import field as dataclass_field
 from typing import Set
 
+from pydantic.dataclasses import dataclass
 from serde import deserialize
 
 from metaphor.bigquery.config import BigQueryRunConfig
