@@ -1,5 +1,5 @@
-from dataclasses import dataclass
-
+from pydantic import validator
+from pydantic.dataclasses import dataclass
 from serde import deserialize
 
 
@@ -13,3 +13,8 @@ class SamplingConfig:
 
     # Sampling only affect table large than threshold
     threshold: int = 100000
+
+    @validator("percentage")
+    def percentage_must_between_1_and_100(cls, v):
+        assert 1 <= v <= 100, "Must be between 1 and 100"
+        return v
